@@ -466,10 +466,6 @@ if algorithms:
                                            salt_size=3,
                                            icv_size=16,
                                            format_mode_iv=_salt_format_mode_iv)
-    # XXX: Flagged as weak by 'cryptography'. Kept for backward compatibility
-    CRYPT_ALGOS['Blowfish'] = CryptAlgo('Blowfish',
-                                        cipher=algorithms.Blowfish,
-                                        mode=modes.CBC)
     # XXX: RFC7321 states that DES *MUST NOT* be implemented.
     # XXX: Keep for backward compatibility?
     # Using a TripleDES cipher algorithm for DES is done by using the same 64
@@ -481,9 +477,6 @@ if algorithms:
     CRYPT_ALGOS['3DES'] = CryptAlgo('3DES',
                                     cipher=algorithms.TripleDES,
                                     mode=modes.CBC)
-    CRYPT_ALGOS['CAST'] = CryptAlgo('CAST',
-                                    cipher=algorithms.CAST5,
-                                    mode=modes.CBC)
 
 ###############################################################################
 if conf.crypto_valid:
