@@ -10,12 +10,20 @@ Routing and handling of network interfaces.
 
 from __future__ import absolute_import
 
-
-import scapy.consts
+from scapy.compat import plain_str
 from scapy.config import conf
 from scapy.error import Scapy_Exception, warning
-from scapy.modules import six
-from scapy.utils import atol, ltoa, itom, plain_str, pretty_list
+from scapy.interfaces import resolve_iface
+from scapy.utils import atol, ltoa, itom, pretty_list
+
+from scapy.compat import (
+    Any,
+    Dict,
+    List,
+    Optional,
+    Tuple,
+    Union,
+)
 
 
 ##############################
@@ -24,36 +32,49 @@ from scapy.utils import atol, ltoa, itom, plain_str, pretty_list
 
 class Route:
     def __init__(self):
+        # type: () -> None
+        self.routes = []  # type: List[Tuple[int, int, str, str, str, int]]
         self.resync()
 
     def invalidate_cache(self):
-        self.cache = {}
+        # type: () -> None
+        self.cache = {}  # type: Dict[str, Tuple[str, str, str]]
 
     def resync(self):
+        # type: () -> None
         from scapy.arch import read_routes
         self.invalidate_cache()
         self.routes = read_routes()
 
     def __repr__(self):
-        rtlst = []
+        # type: () -> str
+        rtlst = []  # type: List[Tuple[Union[str, List[str]], ...]]
         for net, msk, gw, iface, addr, metric in self.routes:
+            if_repr = resolve_iface(iface).description
             rtlst.append((ltoa(net),
                           ltoa(msk),
                           gw,
-                          (iface.description if not isinstance(iface, six.string_types) else iface),  # noqa: E501
+                          if_repr,
                           addr,
                           str(metric)))
 
         return pretty_list(rtlst,
                            [("Network", "Netmask", "Gateway", "Iface", "Output IP", "Metric")])  # noqa: E501
 
-    def make_route(self, host=None, net=None, gw=None, dev=None, metric=1):
+    def make_route(self,
+                   host=None,  # type: Optional[str]
+                   net=None,  # type: Optional[str]
+                   gw=None,  # type: Optional[str]
+                   dev=None,  # type: Optional[str]
+                   metric=1,  # type: int
+                   ):
+        # type: (...) -> Tuple[int, int, str, str, str, int]
         from scapy.arch import get_if_addr
         if host is not None:
             thenet, msk = host, 32
         elif net is not None:
-            thenet, msk = net.split("/")
-            msk = int(msk)
+            thenet, msk_b = net.split("/")
+            msk = int(msk_b)
         else:
             raise Scapy_Exception("make_route: Incorrect parameters. You should specify a host or a net")  # noqa: E501
         if gw is None:
@@ -69,6 +90,7 @@ class Route:
         return (atol(thenet), itom(msk), gw, dev, ifaddr, metric)
 
     def add(self, *args, **kargs):
+        # type: (*Any, **Any) -> None
         """Ex:
         add(net="192.168.1.0/24",gw="1.2.3.4")
         """
@@ -76,6 +98,7 @@ class Route:
         self.routes.append(self.make_route(*args, **kargs))
 
     def delt(self, *args, **kargs):
+        # type: (*Any, **Any) -> None
         """delt(host|net, gw|dev)"""
         self.invalidate_cache()
         route = self.make_route(*args, **kargs)
@@ -83,21 +106,19 @@ class Route:
             i = self.routes.index(route)
             del(self.routes[i])
         except ValueError:
-            warning("no matching route found")
+            raise ValueError("No matching route found!")
 
     def ifchange(self, iff, addr):
+        # type: (str, str) -> None
         self.invalidate_cache()
-        the_addr, the_msk = (addr.split("/") + ["32"])[:2]
-        the_msk = itom(int(the_msk))
+        the_addr, the_msk_b = (addr.split("/") + ["32"])[:2]
+        the_msk = itom(int(the_msk_b))
         the_rawaddr = atol(the_addr)
         the_net = the_rawaddr & the_msk
 
         for i, route in enumerate(self.routes):
             net, msk, gw, iface, addr, metric = route
-            if scapy.consts.WINDOWS:
-                if iff.guid != iface.guid:
-                    continue
-            elif iff != iface:
+            if iff != iface:
                 continue
             if gw == '0.0.0.0':
                 self.routes[i] = (the_net, the_msk, gw, iface, the_addr, metric)  # noqa: E501
@@ -106,26 +127,26 @@ class Route:
         conf.netcache.flush()
 
     def ifdel(self, iff):
+        # type: (str) -> None
         self.invalidate_cache()
         new_routes = []
         for rt in self.routes:
-            if scapy.consts.WINDOWS:
-                if iff.guid == rt[3].guid:
-                    continue
-            elif iff == rt[3]:
+            if iff == rt[3]:
                 continue
             new_routes.append(rt)
         self.routes = new_routes
 
     def ifadd(self, iff, addr):
+        # type: (str, str) -> None
         self.invalidate_cache()
-        the_addr, the_msk = (addr.split("/") + ["32"])[:2]
-        the_msk = itom(int(the_msk))
+        the_addr, the_msk_b = (addr.split("/") + ["32"])[:2]
+        the_msk = itom(int(the_msk_b))
         the_rawaddr = atol(the_addr)
         the_net = the_rawaddr & the_msk
         self.routes.append((the_net, the_msk, '0.0.0.0', iff, the_addr, 1))
 
     def route(self, dst=None, verbose=conf.verb):
+        # type: (Optional[str], int) -> Tuple[str, str, str]
         """Returns the IPv4 routes to a host.
         parameters:
          - dst: the IPv4 of the destination host
@@ -178,16 +199,14 @@ class Route:
         return ret
 
     def get_if_bcast(self, iff):
+        # type: (str) -> List[str]
         bcast_list = []
         for net, msk, gw, iface, addr, metric in self.routes:
             if net == 0:
                 continue    # Ignore default route "0.0.0.0"
             elif msk == 0xffffffff:
                 continue    # Ignore host-specific routes
-            if scapy.consts.WINDOWS:
-                if iff.guid != iface.guid:
-                    continue
-            elif iff != iface:
+            if iff != iface:
                 continue
             bcast = net | (~msk & 0xffffffff)
             bcast_list.append(ltoa(bcast))
@@ -198,12 +217,5 @@ class Route:
 
 conf.route = Route()
 
-iface = conf.route.route(None, verbose=0)[0]
-
-if getattr(iface, "name", iface) == conf.loopback_name:
-    from scapy.arch import get_working_if
-    conf.iface = get_working_if()
-else:
-    conf.iface = iface
-
-del iface
+# Load everything, update conf.iface
+conf.ifaces.reload()
